/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "directorymodel.h"
#include "directoryview.h"
#include "metainfodialog.h"
#include "mpd.h"
#include "mpddirectory.h"
#include <QContextMenuEvent>
#include <QMenu>

DirectoryView::DirectoryView(QWidget *parent) : QTreeView(parent),
		m_midButtonDrag(false),
		m_mpd(MPD::instance()),
m_menu(new QMenu(this)) {
	setObjectName("directoryview");
	setModel(m_model = new DirectoryModel(this));

	// playlist m_menu
	m_enqueueAction = m_menu->addAction(QPixmap(":/icons/add.png"), tr("&Enqueue"), this, SLOT(enqueue()));
	m_playAction = m_menu->addAction(QPixmap(":/icons/play.png"), tr("&Play"), this, SLOT(play()));
	m_menu->addSeparator();
	m_menu->addAction(QPixmap(":/icons/reload.png"), tr("&Rescan directory"), this, SLOT(rescan())); 
	m_menu->addSeparator();
	m_informationAction = m_menu->addAction(QPixmap(":/icons/crystal/info.png"), tr("&Information..."), this, SLOT(information()));

	connect(m_mpd, SIGNAL(directoriesUpdated(MPDDirectory *)), this, SLOT(setRoot(MPDDirectory *)));
	connect(this, SIGNAL(activated(const QModelIndex &)), this, SLOT(activated(const QModelIndex &)));

	toggleActions();
}


void DirectoryView::activated(const QModelIndex &index) {
	if (!index.isValid())
		return;
	QList<MPDDirectory*> dirs;
	dirs << m_model->directory(index);
	m_mpd->addDirectories(dirs, Config::instance()->enqueue());
}

void DirectoryView::contextMenuEvent(QContextMenuEvent *e) {
	QModelIndex clicked = indexAt(e->pos());
	QModelIndexList indexes = selectedIndexes();
	if (indexes.isEmpty() || !indexes.contains(clicked))
		setCurrentIndex(indexAt(e->pos()));
	m_menu->popup(e->globalPos());
}

void DirectoryView::toggleActions() {
	bool t = selectedIndexes().size() > 0;
	m_enqueueAction->setEnabled(t);
	m_informationAction->setEnabled(t);
	m_playAction->setEnabled(t);
}

void DirectoryView::selectionChanged(const QItemSelection &s, const QItemSelection &u) {
	QAbstractItemView::selectionChanged(s, u);
	toggleActions();
	MPDSongList songs;
	foreach (MPDDirectory *d, m_model->directories(selectedIndexes())) {
		songs << d->songs(Config::instance()->recurse());
	}
	emit directoryChanged(songs);
}

void DirectoryView::setRoot(MPDDirectory *dir) {
	m_model->setDirectory(dir);
	// Expand root
	setExpanded(m_model->index(0, 0, QModelIndex()), true);
	setCurrentIndex(m_model->index(0,0));
	toggleActions();
}

void DirectoryView::enqueue() {
	m_mpd->addDirectories(m_model->directories(selectedIndexes()));
}

void DirectoryView::information() {
	QList<MPDDirectory*> dirs = m_model->directories(selectedIndexes());
	MPDSongList songs;
	foreach (MPDDirectory *dir, dirs) {
		songs << dir->songs(Config::instance()->recurse());
	}
	if (!songs.isEmpty())
		new MetaInfoDialog(this, songs);
}

void DirectoryView::play() {
	m_mpd->addDirectories(m_model->directories(selectedIndexes()), false);
}

void DirectoryView::rescan() {
	m_mpd->rescanDirectories(m_model->directories(selectedIndexes()));
}

void DirectoryView::mouseMoveEvent(QMouseEvent *e) {
	QMouseEvent *e2 = e;
	if (m_midButtonDrag)
		e2 = new QMouseEvent(e->type(), e->pos(), Qt::LeftButton, e->buttons() | Qt::LeftButton, e->modifiers());
	QTreeView::mouseMoveEvent(e2);
}

void DirectoryView::mousePressEvent(QMouseEvent *e) {
	QMouseEvent *e2 = e;
	if (e->button() == Qt::MidButton) {
		m_midButtonDrag = true;
		e2 = new QMouseEvent(e->type(), e->pos(), Qt::LeftButton, e->buttons(), e->modifiers());
	}
	QTreeView::mousePressEvent(e2);
}

void DirectoryView::mouseReleaseEvent(QMouseEvent *e) {
	if (e->button() == Qt::MidButton)
		m_midButtonDrag = false;
	QTreeView::mouseReleaseEvent(e);
}
