/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "headerview.h"
#include "mpdsongmodel.h"
#include "mpdsongview.h"

#include <QContextMenuEvent>
#include <QHeaderView>
#include <QMenu>

HeaderView::HeaderView(QWidget *p, View v) : QHeaderView(Qt::Horizontal, p),
		m_config(Config::instance()),
		m_view(v),
m_menu(new QMenu(this)) {
	// Set nice objectName, used for saving config and other neat stuff
	setObjectName(viewName());
	setMovable(true);

	// Create columns list
	if (m_view != PLAYLISTSVIEW && m_view != RADIOVIEW)
		m_columns << TRACK;
	m_columns << TITLE;
	if (m_view != PLAYLISTSVIEW && m_view != RADIOVIEW) {
		m_columns << ARTIST;
		m_columns << ALBUM;
		m_columns << COMPOSER;
		m_columns << DATE;
		m_columns << GENRE;
		m_columns << FILENAME;
		m_columns << LENGTH;
	}
	if (m_view != PLAYLISTSVIEW)
		m_columns << URL;

	autoResizeChanged(m_config->autoResizeColumns());

	connect(m_config, SIGNAL(autoResizeChanged(bool)), this, SLOT(autoResizeChanged(bool)));
}

void HeaderView::mouseReleaseEvent(QMouseEvent *e) {
	// sectionClicked(int) signal is emitted too early, and thus saves the previous sortorder.
	// Therefore we save it here.
	QHeaderView::mouseReleaseEvent(e);
	save();
}

void HeaderView::autoResizeChanged(bool a) {
	if (!a) {
		setResizeMode(QHeaderView::Interactive);
		setStretchLastSection(true);
		return;
	}

	setUpdatesEnabled(false);
	setStretchLastSection(false);
	for (int i=0; i<m_columns.size(); i++) {
		Column col = m_columns.at(i);
		if (col == TRACK || col == LENGTH)
			setResizeMode(i, QHeaderView::ResizeToContents);
		else
			setResizeMode(i, QHeaderView::Stretch);
	}
	setUpdatesEnabled(true);

	if (model())
		update();
}

void HeaderView::restore() {
	disconnect(this, SIGNAL(sectionMoved(int, int, int)), this, SLOT(save()));
	disconnect(this, SIGNAL(sectionResized(int, int, int)), this, SLOT(save()));

	// Restore view state
	QByteArray ba2(m_config->headerViewState(viewName()));
	if (!ba2.isEmpty()) {
		QDataStream ds2(&ba2, QIODevice::ReadOnly);
		for (int j=0;j<model()->columnCount();j++) {
			int size, vi;
			bool hidden;
			ds2 >> size >> vi >> hidden;
			resizeSection(j, size);
			moveSection(visualIndex(j), vi);
			setSectionHidden(j, hidden);
		}
		int sortOrder, sortSection;
		ds2 >> sortOrder >> sortSection;
		setSortIndicator(sortSection, (Qt::SortOrder)sortOrder);
	} else { // Defaults
		int i=0;
		foreach (Column col, m_columns) {
			if (col == HeaderView::TRACK)
				setSectionHidden(i, m_view != HeaderView::PLAYLISTSCONTENTVIEW && m_view != HeaderView::SONGVIEW);
			else if (col == HeaderView::LENGTH)
				setSectionHidden(i, m_view == HeaderView::PLAYLISTSVIEW || m_view == HeaderView::RADIOVIEW);
			else if (col == HeaderView::TITLE)
				setSectionHidden(i, m_view == HeaderView::FILEVIEW);
			else if (col == HeaderView::FILENAME) {
				if (m_view == HeaderView::FILEVIEW)
					setSortIndicator(i, Qt::AscendingOrder);
				setSectionHidden(i, m_view != HeaderView::FILEVIEW);
			} else if (col == HeaderView::URL)
				setSectionHidden(i, m_view != HeaderView::RADIOVIEW);
			else if (col == HeaderView::COMPOSER || col == HeaderView::GENRE || col == HeaderView::DATE)
				setSectionHidden(i, true);
			else { // ARTIST and ALBUM columns
				if (col == HeaderView::ARTIST && m_view == HeaderView::SONGVIEW)
					setSortIndicator(i, Qt::AscendingOrder);
				setSectionHidden(i, m_view != HeaderView::SONGVIEW && m_view != HeaderView::PLAYLISTSCONTENTVIEW);
			}
			i++;
		}
	}

	for (int i=0; i<m_columns.size(); i++) {
		QAction *a = m_menu->addAction(columnName(m_columns.at(i)), this, SLOT(actionTriggered()));
		a->setCheckable(true);
		a->setChecked(!isSectionHidden(i));
	}

	autoResizeChanged(m_config->autoResizeColumns());

	connect(this, SIGNAL(sectionMoved(int, int, int)), this, SLOT(save()));
	connect(this, SIGNAL(sectionResized(int, int, int)), this, SLOT(save()));
}

void HeaderView::save() {
	// Save headerview state
	QByteArray ba;
	QDataStream ds(&ba, QIODevice::ReadWrite);
	for (int i=0;i<model()->columnCount();i++) {
		if (isSectionHidden(i))
			ds << 100;
		else
			ds << sectionSize(i);
		ds << visualIndex(i);
		ds << isSectionHidden(i);
	}
	ds << (int)sortIndicatorOrder();
	ds << sortIndicatorSection();
	m_config->setHeaderViewState(viewName(), ba);
}

void HeaderView::contextMenuEvent(QContextMenuEvent *event) {
	if (m_columns.size() > 1)
		m_menu->popup(event->globalPos());
}

void HeaderView::actionTriggered() {
	int i=0;
	disconnect(this, SIGNAL(sectionMoved(int, int, int)), this, SLOT(save()));
	disconnect(this, SIGNAL(sectionResized(int, int, int)), this, SLOT(save()));
	foreach (QAction *a, m_menu->actions()) {
		setSectionHidden(i++, !a->isChecked());
	}
	connect(this, SIGNAL(sectionMoved(int, int, int)), this, SLOT(save()));
	connect(this, SIGNAL(sectionResized(int, int, int)), this, SLOT(save()));
	save();
}

QList<HeaderView::Column> HeaderView::columns() const {
	return m_columns;
}

QString HeaderView::columnName(Column col) const {
	switch (col) {
	case HeaderView::TITLE:
		if (m_view == HeaderView::PLAYLISTSVIEW)
			return QObject::tr("Playlist");
		if (m_view == HeaderView::RADIOVIEW)
			return QObject::tr("Stream");
		return QObject::tr("Title");
	case HeaderView::TRACK:
		return QObject::tr("Track");
	case HeaderView::ARTIST:
		return QObject::tr("Artist");
	case HeaderView::ALBUM:
		return QObject::tr("Album");
	case HeaderView::LENGTH:
		return QObject::tr("Length");
	case HeaderView::URL:
		return QObject::tr("URL");
	case HeaderView::FILENAME:
		return QObject::tr("Filename");
	case HeaderView::GENRE:
		return QObject::tr("Genre");
	case HeaderView::DATE:
		return QObject::tr("Date");
	case HeaderView::COMPOSER:
		return QObject::tr("Composer");
	default:
		return "?";
	}
}

QString HeaderView::viewName() const {
	switch (m_view) {
	case SONGVIEW:
		return "songview";
	case FILEVIEW:
		return "fileview";
	case RADIOVIEW:
		return "radioview";
	case PLAYLISTSVIEW:
		return "playlistsview";
	case PLAYLISTSCONTENTVIEW:
		return "playlistscontentview";
	default:
		return "UNKNOWNVIEW";
	}
}
