/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "aboutdialog.h"
#include "config.h"
#include "directorypanel.h"
#include "librarypanel.h"
#include "mainwindow.h"
#include "mpd.h"
#include "playlistpanel.h"
#include "playlistspanel.h"
#include "preferencesdialog.h"
#include "radiopanel.h"
#include "richtext.h"
#include "trayicon.h"
#include <QCloseEvent>
#include <QDesktopWidget>
#include <QPainter>
#include <QProgressBar>
#include <QShortcut>

MainWindow::MainWindow() : QMainWindow(0) {
	setupUi(this);
	m_config = Config::instance();
	m_mpd = MPD::instance();

	setWindowTitle(NAMEVER);

	/*
	 * Status bar
	 */
	m_statsLabel = new QLabel();
	m_progressBar = new QProgressBar();
	m_progressBar->hide();
	m_progressBar->setMaximumWidth(100);
	m_progressBar->setMaximumHeight(m_statsLabel->minimumSizeHint().height());
	statusBar()->addPermanentWidget(m_progressBar);
	statusBar()->addPermanentWidget(m_statsLabel);

	// Show program name and version
	statusBar()->showMessage(NAMEVER);
	setStats(MPDStats());

	/*
	 * Ideal style vertical tabbars
	 */
	leftBar->link(leftStack, splitter);
	rightBar->link(rightStack, splitter);
	leftBar->addPanel(new PlaylistPanel(), tr("&Playlist"), QPixmap(":/icons/amarok/playlist.png"), true);
	rightBar->addPanel(new LibraryPanel(), tr("&Library"), QPixmap(":/icons/amarok/collection.png"));
	rightBar->addPanel(new DirectoryPanel(), tr("&Directories"), QPixmap(":/icons/crystal/tree.png"));
	rightBar->addPanel(new RadioPanel(), tr("&Internet Radio"), QPixmap(":/icons/crystal/stream.png"));
	rightBar->addPanel(new PlaylistsPanel(), tr("Pla&ylists"), QPixmap(":/icons/amarok/playlist.png"));

	/*
	 * Signals and slots
	 */
	connect(m_mpd, SIGNAL(statsUpdated(MPDStats)), this, SLOT(setStats(MPDStats)));
	connect(m_mpd, SIGNAL(playingSongUpdated(MPDSong)), this, SLOT(setSong(MPDSong)));
	connect(m_mpd, SIGNAL(disconnected(QString)), this, SLOT(disconnected(QString)));
	connect(m_mpd, SIGNAL(updateStart(int)), this, SLOT(updateStart(int)));
	connect(m_mpd, SIGNAL(updateProgress(int)), this, SLOT(updateProgress(int)));
	connect(m_mpd, SIGNAL(updateDone()), this, SLOT(updateDone()));

	// Menu action signals
	connect(diskonnect, SIGNAL(triggered()), m_mpd, SLOT(disconnectFromMPD()));
	connect(rescanLibrary, SIGNAL(triggered()), m_mpd, SLOT(rescanLibrary()));
	connect(aboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
	connect(quit, SIGNAL(triggered()), qApp, SLOT(quit()));

	// Config signals
	connect(m_config, SIGNAL(alternatingChanged(bool)), this, SLOT(alternatingChanged(bool)));
	connect(m_config, SIGNAL(fontChanged(QFont)), this, SLOT(fontChanged(QFont)));
	connect(m_config, SIGNAL(serverListChanged(ServerInfoList)), this, SLOT(serverListChanged(ServerInfoList)));
	connect(m_config, SIGNAL(opaqueResizeChanged(bool)), this, SLOT(opaqueResizeChanged(bool)));

	/*
	 * Shortcut keys
	 */
	new QShortcut(Qt::Key_Escape, this, SLOT(showHide())); // Escape minimizes to tray

	/*
	 * Tray icon
	 */
	m_trayIcon = new TrayIcon(this);
	if (m_config->trayIconEnabled())
		m_trayIcon->show();
	connect(m_trayIcon, SIGNAL(clicked()), this, SLOT(showHide()));

	/*
	 * Restore state
	 */
	alternatingChanged(m_config->alternatingRowColors());
	serverListChanged(m_config->servers());
	opaqueResizeChanged(m_config->opaqueResize());
	splitter->restore(m_config->mainSplitterSizes());
	setFont(m_config->font());
	qApp->setFont(m_config->font());
	resize(m_config->windowSize());
	if (!m_trayIcon->isVisible() || !m_config->trayIconEnabled() || !m_config->startHidden())
		show();
}

// Auto connected slots
void MainWindow::on_about_triggered() {
	new AboutDialog(this, QPixmap(":/icons/qmpdclient22.png"), NAMEVER);
}

void MainWindow::on_splitter_splitterMoved(int, int) {
	m_config->setMainSplitterSizes(splitter->saveState());
}

void MainWindow::on_preferences_triggered() {
	new PreferencesDialog(this);
}

// User defined slots
void MainWindow::alternatingChanged(bool a) {
	QList<QAbstractItemView*> children = qFindChildren<QAbstractItemView*>(centralWidget(), QString());
	foreach (QObject *child, children) {
		if (!child->setProperty("alternatingRowColors", a))
			qWarning("Couldn't set alternatingRowColors for object: %s", qPrintable(child->objectName()));
	}
}

void MainWindow::closeEvent(QCloseEvent *e) {
	if (m_trayIcon->isVisible() && m_config->trayIconEnabled() && m_config->minimizeToTray()) {
		e->ignore();
		showHide();
		return;
	}
	e->accept();
	qApp->quit();
}

void MainWindow::connectToMPD() {
	QObject *origin = sender();
	if (!origin)
		return;
	QAction *action = qobject_cast<QAction*>(origin);
	m_mpd->connectToMPD(m_config->server(action->text()));
}

void MainWindow::disconnected(const QString &error) {
	if (!error.isEmpty()) {
		statusBar()->showMessage(QString(" %1: %2 ").arg(tr("Error")).arg(error));
		m_trayIcon->showMessage(tr("Error"), error, QSystemTrayIcon::Critical, 5000);
	}
	m_progressBar->reset();
}

void MainWindow::fontChanged(const QFont &font) {
	setFont(font);
	qApp->setFont(font);
}

void MainWindow::opaqueResizeChanged(bool o) {
	splitter->setOpaqueResize(o);
}

void MainWindow::resizeEvent(QResizeEvent *e) {
	m_config->setWindowSize(e->size());
}

void MainWindow::serverListChanged(const ServerInfoList &sil) {
	while (!m_actions.isEmpty())
		delete m_actions.takeLast();
	foreach(ServerInfo si, sil) {
		QAction *a = menu_Connect->addAction(QPixmap(":/icons/crystal/server.png"), si.name, this, SLOT(connectToMPD()));
		if (m_actions.isEmpty())
			a->setShortcut(tr("Ctrl+C"));
		m_actions << a;
	}
}

void MainWindow::setSong(const MPDSong &s) {
	QString windowTitle = NAMEVER;
	if (!s.isNull()) {
		int desktopWidth = QApplication::desktop()->width();
		QString title = elideRichText("", s.title(), "", desktopWidth/2);
		QString artist = elideRichText("", s.artist(), "", desktopWidth/4);
		windowTitle = qApp->applicationName();
		if (!artist.isEmpty())
			windowTitle += " - " + artist;
		windowTitle += " - " + title;
	}
	setWindowTitle(windowTitle);
	setWindowIconText(windowTitle);
}

void MainWindow::setStats(const MPDStats &stats) {
	unsigned long secs = stats.dbPlayTime();
	int day = secs/(60*60*24);
	secs -= day*60*60*24;
	int hour = secs/(60*60);
	secs -= hour*60*60;
	int min = secs/60;
	m_statsLabel->setText(QString(" %1 %2, %3 %4, %5 %6. (%7 %8 %9 %10 %11 %12) ")
	                      .arg(stats.numberOfArtists())
	                      .arg(tr("artists"))
	                      .arg(stats.numberOfAlbums())
	                      .arg(tr("albums"))
	                      .arg(stats.numberOfSongs())
	                      .arg(tr("songs"))
	                      .arg(day)
	                      .arg(tr("days"))
	                      .arg(hour)
	                      .arg(tr("hours"))
	                      .arg(min)
	                      .arg(tr("minutes")));
}

void MainWindow::showHide() {
	if (m_config->trayIconEnabled() && m_config->minimizeToTray() && m_trayIcon->isVisible())
		isVisible() ? hide() : showNormal();
}

void MainWindow::updateStart(int max) {
	m_progressBar->setRange(0, max);
	m_progressBar->setValue(0);
	m_progressBar->show();
	statusBar()->showMessage(tr("Reading library"));
}

void MainWindow::updateProgress(int at) {
	m_progressBar->setValue(at);
}

void MainWindow::updateDone() {
	m_progressBar->setValue(m_progressBar->maximum());
	m_progressBar->hide();
	if (m_mpd->isConnected())
		statusBar()->showMessage(tr("Done reading library"), 2000);
	else
		statusBar()->clearMessage();
}
