/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MPD_H
#define MPD_H

#include <QMap>
#include <QMutex>
#include <QSet>
#include <QStringList>
#include <QThread>
#include "libmpdclient.h"
#include "mpdsonglist.h"
#include "mpdstats.h"
#include "mpdstatus.h"
#include "mpdoutput.h"
#include "serverinfo.h"

class MPD : public QThread {
	Q_OBJECT

private:
	// Connection
	bool m_connected;
	class Config *m_config;
	mpd_Connection *m_connection;
	ServerInfo m_server;
	QMutex m_mutex;

	// Cache
	class MPDDirectory *m_root;
	MPDSongList m_songs, m_playlist;
	QMap<QString, QSet<QString> > m_artistAlbumMap;
	QMap<QString, MPDSongList> m_artistSongMap, m_albumSongMap;
	QMap<MPDSong, MPDSongList> m_playlistMap;
	MPDOutputList m_outputs;

	// State
	QString m_error;
	MPDStats m_stats, m_prevStats;
	MPDStatus m_status, m_prevStatus;
	static MPD *m_instance;

	// Methods
	bool isPaused() const;
	bool isPlaying() const;
	bool finishCommand();
	void reverseMoveAddedSongs(int toPos);
	bool readDirectory(const QString &, MPDDirectory *, MPDDirectory *, MPDSongList *, int *);
	void updateCache();
	void updateCurrentSong();
	void updateOutputDevices();
	void updatePlaylist();
	void updatePlaylists();
	void updateStatus();
	MPD();

private slots:
	void poll();

public:
	~MPD();
	static MPD *instance();
	void run();
	bool isConnected() const;

	// Library
	QStringList albums(const QString &artist) const;
	QStringList artists() const;
	MPDSongList songs() const;
	MPDSongList songsByArtist(const QString &artist) const;
	MPDSongList songsByAlbum(const QString &album, const QString &artist) const;

	// Playlist management
	void addSongs(const MPDSongList &songs, bool enqueue = true, int toPos = -1);
	void addDirectories(const QList<MPDDirectory *> &, bool enqueue = true, int toPos = -1);
	void moveSongs(const MPDSongList &songs, int toPos);
	void removeSongs(const MPDSongList &songs);
	void jumpPlaylistSong(const MPDSong &song);

	// Playlists
	void deletePlaylists(const MPDSongList &playlists);
	void savePlaylist(const QString &playlistName);
	MPDSongList songsInPlaylists(const MPDSongList &songs) const;
	bool playlistExists(const QString &playlistName) const;

	// MPD control
	MPDOutputList outputDevices() const;
	int crossfade() const;

public slots:
	void clearPlaylist();
	void connectToMPD(const ServerInfo &);
	void disconnectFromMPD(bool = true);
	void play();
	void togglePlay();
	void pause();
	void stop();
	void next();
	void prev();
	void rescanDirectories(const QList<MPDDirectory*> &);
	void rescanLibrary();
	void seek(int toSeconds);
	void seekBackward();
	void seekForward();
	void setCrossfade(int seconds);
	void setRandom(bool random);
	void setRepeat(bool repeat);
	void setVolume(int v);
	void shufflePlaylist();
	void toggleOutputDevice(int, bool);
	void volumeUp(int = 5);
	void volumeDown(int = 5);

signals:
	void artistsUpdated(const QStringList &);
	void connected(const ServerInfo &);
	void disconnected(const QString &error);
	void directoriesUpdated(MPDDirectory *);
	void playingSongUpdated(const MPDSong &);
	void playlistUpdated(const MPDSongList &);
	void playlistsUpdated(const MPDSongList &);
	void randomUpdated(bool);
	void repeatUpdated(bool);
	void songsUpdated(const MPDSongList &);
	void stateUpdated(bool);
	void statsUpdated(const MPDStats &);
	void timeUpdated(int, int);
	void updateDone();
	void updateProgress(int);
	void updateStart(int);
	void volumeUpdated(int);
};

#endif
