/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "mpdsong.h"
#include <QDataStream>
#include <QStringList>
#include <QTime>

class MPDSongPrivate {
public:
	MPDSongPrivate();
	void finalize();

	QAtomic ref;
	bool m_null;
	qint32 m_pos, m_secs, m_id;
	MPDSong::Type m_type;
	QString m_album, m_artist, m_track, m_date, m_genre, m_composer, m_url, m_title, m_time;
	QStringList m_haystack;
};

MPDSongPrivate::MPDSongPrivate() :
		ref(1),
		m_null(true),
		m_pos(0),
		m_secs(0),
		m_id(-1),
		m_type(MPDSong::UNKNOWN),
		m_album(QString::null),
		m_artist(QString::null),
		m_track(QString::null),
		m_date(QString::null),
		m_genre(QString::null),
		m_composer(QString::null),
		m_url(QString::null),
		m_title(QString::null),
m_time(QString::null) {}

void MPDSongPrivate::finalize() {
	m_haystack << m_album << m_artist << m_track << m_date << m_genre << m_composer << m_url << m_title << m_time;
}

MPDSong::MPDSong() {
	d = new MPDSongPrivate();
}

MPDSong::MPDSong(const MPDSong &other) {
	d = other.d;
	d->ref.ref();
}

MPDSong::MPDSong(mpd_InfoEntity *entity) {
	d = new MPDSongPrivate();
	if (entity->type == MPD_INFO_ENTITY_TYPE_PLAYLISTFILE)
		parsePlaylist(QString::fromUtf8(entity->info.playlistFile->path));
	else if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
		parseSong(entity->info.song);
	else
		qWarning("MPDSong::MPDSong() unknown entity type");
	d->finalize();
}

MPDSong::~MPDSong() {
	if (!d->ref.deref()) {
		delete d;
		d = 0;
	}
}

/**
 * Create new MPDSong playlist
 * @param url Url of playlist
 * @return MPDSong.
 */
MPDSong MPDSong::createPlaylist(const QString &url) {
	MPDSong song;
	song.d->m_null = false;
	song.parsePlaylist(url);
	return song;
}

/**
 * Create new MPDSong stream.
 * @param title Title of stream.
 * @param url URL of stream.
 * @return MPDSong.
 */
MPDSong MPDSong::createStream(const QString &title, const QString &url) {
	MPDSong song;
	song.d->m_null = false;
	song.d->m_title = title;
	song.d->m_url = url;
	song.d->m_artist = "Various artists";
	song.d->m_album = "Various albums";
	song.d->m_type = STREAM;
	return song;
}

/**
 * Create dummy MPDSong for testing tag-guesser.
 * @param url URL to guess on.
 * @return MPDSong.
 */
MPDSong MPDSong::createTest(const QString &url) {
	MPDSong song;
	song.d->m_null = false;
	song.d->m_url = url;
	return song;
}

// Create song
/**
 * Wrap mpd_Song struct into MPDSong.
 * @param song mpd_Song from libmpdclient.
 * @return MPDSong.
 */
void MPDSong::parseSong(mpd_Song *song) {
	d->m_null = false;
	d->m_url = QString::fromUtf8(song->file);
	d->m_track = QString::fromUtf8(song->track);
	if (song->id > -1 && d->m_url.contains("://"))
		d->m_type = MPDSong::PLAYLISTSTREAM;
	else if (song->id > -1)
		d->m_type = MPDSong::PLAYLISTSONG;
	else
		d->m_type = MPDSong::SONG;
	d->m_pos = song->pos;
	d->m_secs = song->time;
	d->m_id = song->id;
	d->m_title = QString::fromUtf8(song->title);
	d->m_artist = QString::fromUtf8(song->artist);
	d->m_album = QString::fromUtf8(song->album);
	d->m_composer = QString::fromUtf8(song->composer);
	d->m_genre = QString::fromUtf8(song->genre);
	d->m_date = QString::fromUtf8(song->date);
	QTime t = QTime(0, 0).addSecs(d->m_secs);
	d->m_time = t.toString(t.hour() > 0 ? "h:mm:ss" : "m:ss");
}

// Create playlist
/**
 * Wrap mpd_InfoEntity into MPDSong.
 * @param entity mpd_InfoEntity from libmpdclient.
 * @return MPDSong.
 */
void MPDSong::parsePlaylist(const QString &path) {
	d->m_null = false;
	d->m_title = path;
	d->m_url = path;
	d->m_artist = "Playlist (Various artists)";
	d->m_album = "Playlist (Various albums)";
	d->m_type = PLAYLIST;
}

MPDSong::Type MPDSong::type() const {
	return d->m_type;
}

bool MPDSong::isNull() const {
	return d->m_null;
}

int MPDSong::pos() const {
	return d->m_pos;
}

int MPDSong::secs() const {
	return d->m_secs;
}

int MPDSong::id() const {
	return d->m_id;
}

void MPDSong::setArtist(const QString &a) {
	d->m_artist = a;
}

QString MPDSong::artist() const {
	return d->m_artist;
}

void MPDSong::setAlbum(const QString &a) {
	d->m_album = a;
}

QString MPDSong::album() const {
	return d->m_album;
}

void MPDSong::setTrack(const QString &t) {
	d->m_track = t;
}

QString MPDSong::track() const {
	return d->m_track;
}

QString MPDSong::filename() const {
	return d->m_url.section('/', -1, -1);
}

void MPDSong::setUrl(const QString &u) {
	d->m_url = u;
}

QString MPDSong::url() const {
	return d->m_url;
}

/**
 * Get hh:mm formatted time.
 * @return formatted time string.
 */
QString MPDSong::time() const {
	if (d->m_type == STREAM || d->m_url.contains("://"))
		return QString::null;
	return d->m_time;
}

void MPDSong::setTitle(const QString &t) {
	d->m_title = t;
}

QString MPDSong::title() const {
	return d->m_title;
}

QString MPDSong::date() const {
	return d->m_date;
}

QString MPDSong::genre() const {
	return d->m_genre;
}

QString MPDSong::composer() const {
	return d->m_composer;
}

// Check if any of the song's fields contains the regexp
bool MPDSong::contains(const QRegExp &needle) const {
	foreach (QString hay, d->m_haystack) {
		if (!hay.isEmpty() && hay.contains(needle))
			return true;
	}
	return false;
}

/**
 * Warning: this operator just checks if url is less than other url,
 * this should always guarantee a certain sort order, however it
 * might not always be the sort order you want.
 */
bool MPDSong::operator<(const MPDSong &other) const {
	if (url().localeAwareCompare(other.url()) < 0)
		return true;
	return false;
}

/**
 * Equality check
 * @param other MPDSong to compare this one to.
 * @return true if equal, else false.
 */
bool MPDSong::operator==(const MPDSong &other) const {
	if (isNull() && other.isNull())
		return true;
	if (other.isNull())
		return false;
	return (d->m_url == other.url()
	        && d->m_track == other.track()
	        && d->m_secs == other.secs()
	        && d->m_title == other.title()
	        && d->m_artist == other.artist()
	        && d->m_album == other.album()
	        && d->m_composer == other.composer()
	        && d->m_genre == other.genre()
	        && d->m_date == other.date()
	        && d->m_type == other.type()
	        && d->m_id == other.id()
	        && d->m_pos == other.pos());
}

MPDSong &MPDSong::operator=(const MPDSong &other) {
	qAtomicAssign(d, other.d);
	return *this;
}

QDataStream &operator<<(QDataStream &s, const MPDSong &p) {
	s << (qint32)p.isNull();
	s << p.pos();
	s << p.secs();
	s << p.id();
	s << (qint32)p.type();
	s << p.album();
	s << p.artist();
	s << p.track();
	s << p.date();
	s << p.genre();
	s << p.composer();
	s << p.url();
	s << p.title();
	s << p.time();
	return s;
}

QDataStream &operator>>(QDataStream &s, MPDSong &p) {
	qint32 iin;
	QString sin;
	s >> iin;
	p.d->m_null = (bool)iin;
	s >> iin;
	p.d->m_pos = iin;
	s >> iin;
	p.d->m_secs = iin;
	s >> iin;
	p.d->m_id = iin;
	s >> iin;
	p.d->m_type = (MPDSong::Type)iin;
	s >> sin;
	p.d->m_album = sin;
	s >> sin;
	p.d->m_artist = sin;
	s >> sin;
	p.d->m_track = sin;
	s >> sin;
	p.d->m_date = sin;
	s >> sin;
	p.d->m_genre = sin;
	s >> sin;
	p.d->m_composer = sin;
	s >> sin;
	p.d->m_url = sin;
	s >> sin;
	p.d->m_title = sin;
	s >> sin;
	p.d->m_time = sin;
	return s;
}
