/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "mpdsongmodel.h"
#include <QMimeData>

MPDSongModel::MPDSongModel(QObject *parent) : QAbstractTableModel(parent), m_dragDisabled(false) {
	setObjectName("mpdsongmodel");
}

MPDSongModel::MPDSongModel(QObject *parent, HeaderView *hv) : QAbstractTableModel(parent),
		m_dragDisabled(false),
		m_header(hv),
m_columns(hv->columns()) {
	setObjectName("mpdsongmodel");
}

QVariant MPDSongModel::data(const QModelIndex &index, int role) const {
	if (!index.isValid() || index.row() < 0 || index.row() >= m_visible.size() || index.column() >= m_columns.size())
		return QVariant();

	MPDSong song = m_visible.at(index.row());
	if (song.isNull())
		return QVariant();

	if (role == Qt::DisplayRole) {
		switch (m_columns.at(index.column())) {
		case HeaderView::TRACK:
			return song.track();
		case HeaderView::TITLE:
			return song.title();
		case HeaderView::LENGTH:
			return song.time();
		case HeaderView::ALBUM:
			return song.album();
		case HeaderView::ARTIST:
			return song.artist();
		case HeaderView::FILENAME:
			return song.filename();
		case HeaderView::URL:
			return song.url();
		case HeaderView::GENRE:
			return song.genre();
		case HeaderView::DATE:
			return song.date();
		case HeaderView::COMPOSER:
			return song.composer();
		default:
			return "?";
		}
	}

	if (role == Qt::TextAlignmentRole && m_columns.at(index.column()) == HeaderView::LENGTH)
		return Qt::AlignRight;

	return QVariant();
}

QVariant MPDSongModel::headerData(int section, Qt::Orientation orientation, int role) const {
	if (role != Qt::DisplayRole || section >= m_columns.size() || section < 0 || orientation != Qt::Horizontal)
		return QVariant();
	return m_header->columnName(m_columns.at(section));
}

void MPDSongModel::setAlbumFilter(const QRegExp &a) {
	m_albumFilter = a;
	filter();
}

void MPDSongModel::setArtistFilter(const QRegExp &a) {
	m_artistFilter = a;
	filter();
}

void MPDSongModel::setFilter(const QString &a) {
	m_titleFilter = QRegExp(a, Qt::CaseInsensitive);
	filter();
}

int MPDSongModel::columnCount(const QModelIndex &) const {
	return m_columns.size();
}

int MPDSongModel::rowCount(const QModelIndex &) const {
	return m_visible.size();
}

Qt::ItemFlags MPDSongModel::flags(const QModelIndex &) const {
	return Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
}

void MPDSongModel::setSongs(const MPDSongList &s) {
	m_all = s;
	filter();
}

MPDSong MPDSongModel::song(const QModelIndex &index) const {
	if (!index.isValid() || index.row() < 0 || m_visible.count() <= index.row())
		return 0;
	return m_visible.at(index.row());
}

MPDSongList MPDSongModel::songs() const {
	return m_visible;
}

MPDSongList MPDSongModel::songs(const QModelIndex &index) const {
	QModelIndexList indexes;
	indexes.append(index);
	return songs(indexes);
}

MPDSongList MPDSongModel::songs(const QModelIndexList &indexes) const {
	MPDSongList songs;
	foreach (QModelIndex index, indexes) {
		MPDSong s = song(index);
		if (s.isNull() || songs.contains(s))
			continue;
		songs << s;
	}
	return songs;
}

void MPDSongModel::filter() {
	if (m_titleFilter.isEmpty()
	        && m_artistFilter.isEmpty()
	        && m_albumFilter.isEmpty()) {
		m_visible = m_all;
	} else {
		m_visible.clear();
		foreach (MPDSong s, m_all) {
			if (!m_artistFilter.isEmpty() && !m_artistFilter.exactMatch(s.artist()))
				continue;
			if (!m_albumFilter.isEmpty() && !m_albumFilter.exactMatch(s.album()))
				continue;

			if (m_titleFilter.isEmpty() || s.contains(m_titleFilter))
				m_visible << s;
		}
	}
	reset();
}

void MPDSongModel::sort(int column, Qt::SortOrder order) {
	if (column < 0 || column >= m_columns.size())
		return;

	m_all.sort(m_columns.at(column), order);
	filter();
}

/**
 * DND
 */
void MPDSongModel::setDragDisabled(bool n) {
	m_dragDisabled = n;
}

bool MPDSongModel::isDragDisabled() const {
	return m_dragDisabled;
}

QStringList MPDSongModel::mimeTypes() const {
	QStringList types;
	types << "qmpdclient/song" << "qmpdclient/playlist";
	return types;
}

QMimeData *MPDSongModel::mimeData(const QModelIndexList &indexes) const {
	if (m_dragDisabled)
		return 0;
	QByteArray encodedData;
	QDataStream stream(&encodedData, QIODevice::WriteOnly);
	foreach (MPDSong song, songs(indexes)) {
		stream << song;
	}

	QMimeData *mimeData = new QMimeData();
	mimeData->setData("qmpdclient/song", encodedData);
	return mimeData;
}
