/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "jumptosongdialog.h"
#include "metainfodialog.h"
#include "mpd.h"
#include "playlistitemdelegate.h"
#include "playlistmodel.h"
#include "playlistview.h"
#include <QContextMenuEvent>
#include <QInputDialog>
#include <QMenu>
#include <QPainter>

PlaylistView::PlaylistView(QWidget *parent) : QListView(parent),
		m_config(Config::instance()),
		m_mpd(MPD::instance()),
m_menu(new QMenu(this)) {
	setItemDelegate(new PlaylistDelegate(this));
	setModel(m_model = new PlaylistModel(this));
	m_model->setPattern(m_config->playlistPattern());

	// playlist menu
	QAction *jumpToAction = m_menu->addAction(QPixmap(":/icons/crystal/goto.png"), tr("J&ump to song..."), this, SLOT(jumpToSong()), tr("Ctrl+U", "This is the shortcut for 'Jump to song in playlist'"));

	addAction(jumpToAction);
	m_menu->addSeparator();
	m_removeAction = m_menu->addAction(QPixmap(":/icons/remove.png"), tr("&Remove"), this, SLOT(removeItems()));
	m_removeAction->setShortcut(Qt::Key_Delete);
	m_removeAction->setShortcutContext(Qt::WidgetShortcut);
	addAction(m_removeAction);
	m_cropAction = m_menu->addAction(QPixmap(":/icons/crop.png"), tr("&Crop playlist"), this, SLOT(cropItems()));
	m_menu->addAction(QPixmap(":/icons/removeall.png"), tr("Remove &all"), m_mpd, SLOT(clearPlaylist()));
	m_menu->addSeparator();
	m_randomAction = m_menu->addAction(QPixmap(":/icons/amarok/random.png"), tr("Ra&ndom play"));
	m_repeatAction = m_menu->addAction(QPixmap(":/icons/amarok/repeat.png"), tr("Re&peat play"));
	m_menu->addAction(QPixmap(":/icons/amarok/shuffle.png"), tr("S&huffle playlist"), m_mpd, SLOT(shufflePlaylist()));
	m_menu->addSeparator();
	m_menu->addAction(QPixmap(":/icons/crystal/saveas.png"), tr("&Save playlist..."), this, SLOT(savePlaylist()));
	m_informationAction = m_menu->addAction(QPixmap(":/icons/crystal/info.png"), tr("&Information..."), this, SLOT(information()));
	m_randomAction->setCheckable(true);
	m_repeatAction->setCheckable(true);

	// signals and slots connections
	connect(m_mpd, SIGNAL(playingSongUpdated(MPDSong)), this, SLOT(setSong(MPDSong)));
	connect(m_mpd, SIGNAL(playlistUpdated(MPDSongList)), this, SLOT(setPlaylist(MPDSongList)));
	connect(m_mpd, SIGNAL(randomUpdated(bool)), m_randomAction, SLOT(setChecked(bool)));
	connect(m_mpd, SIGNAL(repeatUpdated(bool)), m_repeatAction, SLOT(setChecked(bool)));
	connect(m_randomAction, SIGNAL(toggled(bool)), m_mpd, SLOT(setRandom(bool)));
	connect(m_repeatAction, SIGNAL(toggled(bool)), m_mpd, SLOT(setRepeat(bool)));
	connect(this, SIGNAL(activated(const QModelIndex &)), this, SLOT(indexActivated(const QModelIndex &)));

	selectionChanged();
}

void PlaylistView::selectionChanged(const QItemSelection &s, const QItemSelection &u) {
	QListView::selectionChanged(s, u);
	bool t = selectedIndexes().count() > 0;
	m_cropAction->setEnabled(t);
	m_informationAction->setEnabled(t);
	m_removeAction->setEnabled(t);
	emit toggleActions(t);
}


void PlaylistView::indexActivated(const QModelIndex &index) {
	foreach(MPDSong s, m_model->songs(index)) {
		m_mpd->jumpPlaylistSong(s);
		break;
	}
}

void PlaylistView::cropItems() {
	m_mpd->removeSongs(m_model->unSelectedSongs(selectedIndexes()));
}

void PlaylistView::removeItems() {
	QModelIndexList selected = selectedIndexes();
	if (selected.isEmpty())
		return;
	m_mpd->removeSongs(m_model->songs(selected));
}

void PlaylistView::setPlaylist(const MPDSongList &songs) {
	QModelIndexList selected = selectedIndexes();
	QModelIndex last = selected.isEmpty() ? QModelIndex() : selected.last();
	m_model->setSongs(songs);
	setCurrentIndex(last);
	selectionChanged();
}

void PlaylistView::setSong(const MPDSong &s) {
	m_model->setPlaying(s);
	if (!s.isNull() && m_config->scrollToPlaying()) {
		scrollTo(m_model->indexOfSong(s), QAbstractItemView::PositionAtCenter);
		m_hidden = true;
	}
}

void PlaylistView::hideEvent(QHideEvent *e) {
	QListView::hideEvent(e);
	m_hidden = true;
}
void PlaylistView::paintEvent(QPaintEvent *e) {
	QListView::paintEvent(e);
	if (!m_dropLine.isNull()) {
		QPainter painter(viewport());
		painter.drawLine(m_dropLine);
	}
	if (m_hidden && m_config->scrollToPlaying()) {
		scrollTo(m_model->indexOfSong(m_model->playingSong()), QAbstractItemView::PositionAtCenter);
		m_hidden = false;
	}
}

void PlaylistView::savePlaylist() {
	bool ok;
	QString plName = QInputDialog::getText(this, tr("Save playlist as..."),
	                                       tr("Playlist name:"), QLineEdit::Normal, "", &ok);
	while (ok && !plName.isEmpty()) {
		if (!m_mpd->playlistExists(plName)) {
			m_mpd->savePlaylist(plName);
			break;
		}
		QString msg = tr("A playlist with that name already exists.\nPlease use another name:");
		plName = QInputDialog::getText(this, tr("Save playlist as..."),
		                               msg, QLineEdit::Normal, "", &ok);
	}
}

void PlaylistView::jumpToSong() {
	JumpToSongDialog *search = new JumpToSongDialog(this, m_model->songs(), m_model->playingSong());
	if (search->exec() == QDialog::Accepted) {
		m_mpd->jumpPlaylistSong(search->jumpSong());
		if (m_config->scrollToPlaying())
			scrollTo(m_model->indexOfSong(search->jumpSong()), QAbstractItemView::PositionAtCenter);
	}
	delete search;
}

void PlaylistView::information() {
	new MetaInfoDialog(this, m_model->songs(selectedIndexes()));
}

void PlaylistView::contextMenuEvent(QContextMenuEvent *e) {
	QModelIndex clicked = indexAt(e->pos());
	QModelIndexList indexes = selectedIndexes();
	if (indexes.isEmpty() || !indexes.contains(clicked))
		setCurrentIndex(indexAt(e->pos()));
	m_menu->popup(e->globalPos());
}

// Hacks to enable contigous selections with middle mouse button.
void PlaylistView::mouseMoveEvent(QMouseEvent *e) {
	QMouseEvent *e2 = e;
	if (m_model->isDragDisabled())
		e2 = new QMouseEvent(e->type(), e->pos(), Qt::LeftButton, e->buttons() | Qt::LeftButton, e->modifiers());
	QListView::mouseMoveEvent(e2);
}

void PlaylistView::mousePressEvent(QMouseEvent *e) {
	QMouseEvent *e2 = e;
	if (e->button() == Qt::MidButton) {
		m_model->setDragDisabled(true);
		e2 = new QMouseEvent(e->type(), e->pos(), Qt::LeftButton, e->buttons(), e->modifiers());
	}
	QListView::mousePressEvent(e2);
}

void PlaylistView::mouseReleaseEvent(QMouseEvent *e) {
	if (e->button() == Qt::MidButton)
		m_model->setDragDisabled(false);
	QListView::mouseReleaseEvent(e);
}

// Set up custom drop indicator, since the default one does not behave as I wish.
void PlaylistView::dragMoveEvent(QDragMoveEvent *e) {
	QListView::dragMoveEvent(e);
	if (m_model->rowCount() < 1) // The list is empty, draw nothing
		return;

	startAutoScroll();

	QModelIndex mouseIndex = indexAt(e->pos());
	if (!mouseIndex.isValid())
		mouseIndex = m_model->index(m_model->rowCount()-1, 0);

	QRect itemRect = visualRect(mouseIndex);

	int y = itemRect.bottom();
	int halfy = itemRect.y() + itemRect.height()/2;

	if (mouseIndex.row() == 0 && e->pos().y() < halfy) {
		// We are at first song, and halfway above item, draw line on top
		y = itemRect.top();
		m_model->setTopDrop(true);
	} else
		m_model->setTopDrop(false);

	m_dropLine = QLine(0, y, viewport()->width(), y);
	setDirtyRegion(visibleRegion());
	repaint();
}

// Remove drop indicator
void PlaylistView::dropEvent(QDropEvent *e) {
	QListView::dropEvent(e);
	m_dropLine = QLine();
	setDirtyRegion(visibleRegion());
	repaint();
}

// Remove drop indicator
void PlaylistView::dragLeaveEvent(QDragLeaveEvent *e) {
	QListView::dragLeaveEvent(e);
	m_dropLine = QLine();
	setDirtyRegion(visibleRegion());
	repaint();
}
