/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "mpd.h"
#include "mpdsongmodel.h"
#include "passivepopup.h"
#include "preferencesdialog.h"
#include "servermodel.h"
#include "tagguesser.h"
#include <QDir>
#include <QFontDialog>
#include <QItemDelegate>
#include <QKeyEvent>

// Work around bug in Qt 4.2.[0-1]. Remove at a later date.
class ItemDelegate : public QItemDelegate {
	bool eventFilter(QObject *object, QEvent *event) {
		QWidget *editor = ::qobject_cast<QWidget*>(object);
		if (!editor)
			return QItemDelegate::eventFilter(object, event);
		if (event->type() == QEvent::KeyPress) {
			switch (static_cast<QKeyEvent *>(event)->key()) {
			case Qt::Key_Enter:
			case Qt::Key_Return:
				emit commitData(editor);
				emit closeEditor(editor, QAbstractItemDelegate::SubmitModelCache);
				return true;
			}
		}
		return QItemDelegate::eventFilter(object, event);
	}
};

PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
	setupUi(this);
	m_config = Config::instance();
	m_mpd = MPD::instance();
	setWindowTitle(tr("Preferences"));
	setAttribute(Qt::WA_DeleteOnClose, true);

	splitter->setStretchFactor(0, 1);
	splitter->setStretchFactor(1, 10);
	categoryList->header()->hide();

	// Connection
	serverList->setModel(m_serverModel = new ServerModel(this));
	serverList->setItemDelegate(new ItemDelegate());
	m_serverSel = serverList->selectionModel();
	connect(m_serverSel, SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)),
	        this, SLOT(serverSelectionChanged()));
	m_serverModel->setServers(m_config->servers());
	deleteButton->setEnabled(false);
	upButton->setEnabled(false);
	downButton->setEnabled(false);
	autoConnectCheck->setChecked(m_config->autoconnect());
	timeoutSpin->setValue(m_config->timeoutTime());
	reconnectCheck->setChecked(m_config->reconnect());
	reconnectSpin->setValue(m_config->reconnectTime());

	// Server settings
	crossfadeCheck->setChecked(m_mpd->crossfade() > 0);
	crossfadeSpin->setValue(m_mpd->crossfade());
	foreach (MPDOutput o, m_mpd->outputDevices()) {
		QTreeWidgetItem *i = new QTreeWidgetItem(outputList, o.id());
		i->setText(1, o.name());
		i->setCheckState(0, o.enabled() ? Qt::Checked : Qt::Unchecked);
	}

	// Tray icon
	trayIconCheck->setChecked(m_config->trayIconEnabled());
	startHiddenCheck->setChecked(m_config->startHidden());
	minToTrayCheck->setChecked(m_config->minimizeToTray());

	// Notifications
	PassivePopup::Position pos = (PassivePopup::Position)m_config->notificationsPosition();
	m_positionGroup = new QButtonGroup(this);
	m_positionGroup->addButton(topLeft, PassivePopup::TopLeft);
	m_positionGroup->addButton(top, PassivePopup::Top);
	m_positionGroup->addButton(topRight, PassivePopup::TopRight);
	m_positionGroup->addButton(right, PassivePopup::Right);
	m_positionGroup->addButton(bottomRight, PassivePopup::BottomRight);
	m_positionGroup->addButton(bottom, PassivePopup::Bottom);
	m_positionGroup->addButton(bottomLeft, PassivePopup::BottomLeft);
	m_positionGroup->addButton(left, PassivePopup::Left);

	topLeft->setChecked(pos == PassivePopup::TopLeft);
	top->setChecked(pos == PassivePopup::Top);
	topRight->setChecked(pos == PassivePopup::TopRight);
	right->setChecked(pos == PassivePopup::Right);
	bottom->setChecked(pos == PassivePopup::Bottom);
	bottomLeft->setChecked(pos == PassivePopup::BottomLeft);
	left->setChecked(pos == PassivePopup::Left);
	bottomRight->setChecked(pos == PassivePopup::BottomRight);

	notificationsTimeoutSpinner->setValue(m_config->notificationsTimeout());
	notificationsCheck->setChecked(m_config->notificationsEnabled());

	// Look and feel
	setFontString(font());
	if (m_config->enqueue())
		enqueueItemRadio->setChecked(true);
	else
		playItemRadio->setChecked(true);
	autoResizeCheck->setChecked(m_config->autoResizeColumns());
	recursiveViewCheck->setChecked(m_config->recurse());
	opaqueCheck->setChecked(m_config->opaqueResize());
	alternatingCheck->setChecked(m_config->alternatingRowColors());

	// Look and feel playlist
	QString help = "<table width='100%'>";
	help += QString("<tr><td>%t: %1</td><td>%d: %2</td></tr>").arg(tr("Title")).arg(tr("Date"));
	help += QString("<tr><td>%b: %1</td><td>%c: %2</td></tr>").arg(tr("Artist")).arg(tr("Composer"));
	help += QString("<tr><td>%a: %1</td><td>%f: %2</td></tr>").arg(tr("Album")).arg(tr("Filename"));
	help += QString("<tr><td>%n: %1</td><td>%u: %2</td></tr>").arg(tr("Track")).arg(tr("URL"));
	help += QString("<tr><td>%g: %1</td><td><br></td></tr>").arg(tr("Genre"));
	help += "</table>";
	titleFormatLabel->setText(help);
	titleFormatLine->setText(m_config->playlistPattern());
	centerPlayingCheck->setChecked(m_config->scrollToPlaying());

	// Language
	translationCheck->setChecked(m_config->translation());
	// Find all translations
	QDir dir(":/translations/");
	dir.setFilter(QDir::Files);
	QFileInfoList list = dir.entryInfoList();
	for (int i = 0; i < list.size(); ++i) {
		QFileInfo fileInfo = list.at(i);
		localeList->addItem(fileInfo.baseName().section("_", 1, 1));
	}

	QList<QListWidgetItem *>foundLocales = localeList->findItems(m_config->locale(), Qt::MatchExactly);
	if (foundLocales.isEmpty())
		localeList->setCurrentRow(0);
	else
		localeList->setCurrentItem(foundLocales.first());

	// Tagguesser
	tagGuesserCheck->setChecked(m_config->tagGuesserEnabled());
	patternLine->setText(m_config->guessPattern());
	testLine->setText(m_config->testFilename());
	m_origPattern = m_config->guessPattern();

	// Auto player
	autoAddCheck->setChecked(m_config->autoAddSongs());
	autoAddCountSpin->setValue(m_config->autoAddCount());
	autoAddPosSpin->setValue(m_config->autoAddPos());
	autoRemoveCheck->setChecked(m_config->autoRemoveSongs());

	// Views
	widgetStack->setCurrentIndex(0);
	for (int i=0, index=0; i<categoryList->topLevelItemCount(); i++, index++) {
		QTreeWidgetItem *item = categoryList->topLevelItem(i);
		item->setExpanded(true);
		m_relation.insert(item, index);
		for (int j=0; j<item->childCount(); j++)
			m_relation.insert(item->child(j), ++index);
	}
	categoryList->setCurrentItem(categoryList->topLevelItem(0));

	show();

	for (int i=0; i<m_serverModel->columnCount(); i++)
		serverList->resizeColumnToContents(i);

	alternatingChanged();
}

void PreferencesDialog::alternatingChanged() {
	QList<QAbstractItemView*> children = qFindChildren<QAbstractItemView*>(this, QString());
	foreach (QObject *child, children) {
		if (!child->setProperty("alternatingRowColors", alternatingCheck->isChecked()))
			qWarning("Couldn't set alternatingRowColors for object: %s", qPrintable(child->objectName()));
	}
}

void PreferencesDialog::on_applyButton_clicked() {
	// Server
	m_config->setServers(m_serverModel->servers());

	m_config->setTimeoutTime(timeoutSpin->value());
	m_config->setAutoconnect(autoConnectCheck->isChecked());
	m_config->setReconnect(reconnectCheck->isChecked());
	m_config->setReconnectTime(reconnectSpin->value());

	// Server settings
	if (crossfadeSpin->value() > 0 && crossfadeCheck->isChecked())
		m_mpd->setCrossfade(crossfadeSpin->value());
	else
		m_mpd->setCrossfade(0);
	foreach (QTreeWidgetItem *i, outputList->findItems("*", Qt::MatchWildcard)) {
		m_mpd->toggleOutputDevice(i->type(), i->checkState(0) == Qt::Checked ? true : false);
	}

	// Look and feel
	m_config->setAlternatingRowColors(alternatingCheck->isChecked());
	m_config->setEnqueue(enqueueItemRadio->isChecked());
	m_config->setFont(fontLine->font());
	m_config->setAutoResizeColumns(autoResizeCheck->isChecked());
	m_config->setOpaqueResize(opaqueCheck->isChecked());
	m_config->setRecurse(recursiveViewCheck->isChecked());
	// Playlist
	m_config->setScrollToPlaying(centerPlayingCheck->isChecked());
	m_config->setPlaylistPattern(titleFormatLine->text());

	// Language
	m_config->setTranslation(translationCheck->isChecked());
	QList<QListWidgetItem*> locales = localeList->selectedItems();
	if (locales.isEmpty() || locales.first() == localeList->item(0))
		m_config->setLocale("");
	else
		m_config->setLocale(locales.first()->text());

	// Tag guesser
	m_config->setTagGuesserEnabled(tagGuesserCheck->isChecked());
	m_config->setGuessPattern(patternLine->text());
	m_config->setTestFilename(testLine->text());

	// Trayicon
	m_config->setTrayIconEnabled(trayIconCheck->isChecked());
	m_config->setMinimizeToTray(minToTrayCheck->isChecked());
	m_config->setStartHidden(startHiddenCheck->isChecked());

	// Auto player
	m_config->setAutoAddSongs(autoAddCheck->isChecked());
	m_config->setAutoAddPos(autoAddPosSpin->value());
	m_config->setAutoAddCount(autoAddCountSpin->value());
	m_config->setAutoRemoveSongs(autoRemoveCheck->isChecked());

	// Notifications
	m_config->setNotificationsPosition(m_positionGroup->checkedId());
	m_config->setNotificationsTimeout(notificationsTimeoutSpinner->value());
	m_config->setNotificationsEnabled(notificationsCheck->isChecked());

	alternatingChanged();
}

void PreferencesDialog::on_okButton_clicked() {
	on_applyButton_clicked();
	close();
}

void PreferencesDialog::on_cancelButton_clicked() {
	// Revert to original guess pattern
	m_config->setGuessPattern(m_origPattern);
	close();
}

void PreferencesDialog::on_fontButton_clicked() {
	bool ok;
	QFont newfont = QFontDialog::getFont(&ok, fontButton->font(), this);
	if (ok) {
		setFontString(newfont);
		emit resizeEvent(0);
	}
}

void PreferencesDialog::setFontString(const QFont &newfont) {
	QString font = newfont.family();
	if (newfont.bold())
		font += QString(", %1").arg(tr("bold"));
	if (newfont.italic())
		font += QString(", %1").arg(tr("italic"));
	font += QString(", %1").arg(newfont.pointSize());
	fontLine->setText(font);
	fontLine->setFont(newfont);
}

void PreferencesDialog::on_testLine_textChanged(const QString &url) {
	if (url.isEmpty())
		return;
	// Create song to guess on.
	MPDSong song = MPDSong::createTest(url);
	TagGuesser::guessTags(song);
	bandLabel->setText(QString("%1: %2").arg(tr("Artist")).arg(song.artist()));
	albumLabel->setText(QString("%1: %2").arg(tr("Album")).arg(song.album()));
	trackLabel->setText(QString("%1: %2").arg(tr("Track")).arg(song.track()));
	titleLabel->setText(QString("%1: %2").arg(tr("Title")).arg(song.title()));
}

void PreferencesDialog::on_patternLine_textChanged(const QString &pattern) {
	m_config->setGuessPattern(pattern);
	if (pattern.isEmpty())
		return;
	on_testLine_textChanged(testLine->text());
}

void PreferencesDialog::on_categoryList_currentItemChanged(QTreeWidgetItem *c, QTreeWidgetItem *) {
	widgetStack->setCurrentIndex(m_relation.value(c));
	currentLabel->setText("<h3>" + c->text(0) + "</h3>");
}

void PreferencesDialog::on_addButton_clicked() {
	ServerInfoList sil = m_serverModel->servers();
	ServerInfo si("Edit me");
	sil << si;
	m_serverModel->setServers(sil);
}

void PreferencesDialog::on_deleteButton_clicked() {
	foreach (QModelIndex sel, m_serverSel->selectedIndexes()) {
		m_serverModel->deleteServer(sel);
		break;
	}
	deleteButton->setEnabled(m_serverModel->size() > 1 && !m_serverSel->selectedIndexes().isEmpty());
}

void PreferencesDialog::serverSelectionChanged() {
	deleteButton->setEnabled(m_serverModel->size() > 1 && !m_serverSel->selectedIndexes().isEmpty());
	foreach (QModelIndex sel, m_serverSel->selectedIndexes()) {
		upButton->setEnabled(sel.row() > 0);
		downButton->setEnabled(sel.row() < m_serverModel->size()-1);
		return;
	}
	upButton->setEnabled(false);
	downButton->setEnabled(false);
}

void PreferencesDialog::on_downButton_clicked() {
	foreach (QModelIndex sel, m_serverSel->selectedIndexes()) {
		if (m_serverModel->moveDown(sel)) {
			m_serverSel->setCurrentIndex(m_serverModel->index(sel.row()+1, 0),
			                             QItemSelectionModel::Clear);
			m_serverSel->setCurrentIndex(m_serverModel->index(sel.row()+1, 0),
			                             QItemSelectionModel::Select | QItemSelectionModel::Rows);
		}
		return;
	}
}

void PreferencesDialog::on_upButton_clicked() {
	foreach (QModelIndex sel, m_serverSel->selectedIndexes()) {
		if (m_serverModel->moveUp(sel)) {
			m_serverSel->setCurrentIndex(m_serverModel->index(sel.row()-1, 0),
			                             QItemSelectionModel::Clear);
			m_serverSel->setCurrentIndex(m_serverModel->index(sel.row()-1, 0),
			                             QItemSelectionModel::Select | QItemSelectionModel::Rows);
		}
		return;
	}
}
