/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "config.h"
#include "dynamicplaylist.h"
#include "mainwindow.h"
#include "mpd.h"
#include "notifications.h"
#include "qmpdclient.h"
#include "reconnect.h"
#include <QLibraryInfo>
#include <QTranslator>
#include <QWheelEvent>

QMPDClient::QMPDClient(int &argc, char **argv) : QApplication(argc, argv) {
	setObjectName("qmpdclient");
	setApplicationName("QMPDClient");
	setOrganizationName("QMPDClient");

	m_config = Config::instance();
	// Translation
	if (m_config->translation()) {
		QTranslator *translator = new QTranslator(this);
		QTranslator *qtTranslator = new QTranslator(this);
		if (translator->load(QString("qmpdclient_")+m_config->locale(), ":/translations/", QString(), ".qm"))
			installTranslator(translator);
		if (qtTranslator->load(QString("qt_")+m_config->locale(), QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
			installTranslator(qtTranslator);
	}

	m_mpd = MPD::instance();
	m_mpd->start();

	// Start services
	new DynamicPlaylist(this);
	new Reconnect(this);
#ifdef Q_WS_X11
	grabKeys();
#endif

	// Create mainwin
	m_mainWindow = new MainWindow();
	connect(this, SIGNAL(lastWindowClosed()), this, SLOT(quit()));
	new Notifications(this, m_mainWindow->winId());

	if (m_config->autoconnect()) // Autoconnect?
		m_mpd->connectToMPD(m_config->servers().first());

	// Install event filter to pick up wheel over tray icon
	installEventFilter(this);
}

QMPDClient::~QMPDClient() {
	// Stop MPD-thread's event loop
	m_mpd->quit();

	// Stopping MPD takes a while, do something useful meanwhile.
#ifdef Q_WS_X11
	ungrabKeys();
#endif
	delete m_mainWindow;
	delete m_config;

	// Make sure MPD is stopped before delete
	m_mpd->wait();
	delete m_mpd;
}

/**
 * This eventFilter is used to detect wheelEvent over the tray icon.
 * If the object which recieves the wheelEvent has no parent it is
 * either the main window or the tray icon. So, if the wheelEvents
 * position is outside the mainwindow frame, it must be the tray icon.
 */
bool QMPDClient::eventFilter(QObject *watched, QEvent *event) {
	if (event->type() == QEvent::Wheel) {
		QWheelEvent *e = static_cast<QWheelEvent*>(event);
		if (!watched->parent() && !m_mainWindow->frameGeometry().contains(e->globalPos())) {
			// No parent, and outside mainwindow, must be trayicon.
			int numDegrees = e->delta() / 8;
			int numSteps = numDegrees / 15;
			if (e->orientation() == Qt::Vertical) {
				if (numSteps > 0)
					m_mpd->volumeUp(5*numSteps);
				else if (numSteps < 0)
					m_mpd->volumeDown(5*numSteps);
				return true;
			}
		}
	}
	return QApplication::eventFilter(watched, event);
}

/*
 * X11 only multimedia keys support
 */
#ifdef Q_WS_X11
#include <QDesktopWidget>
#include <QX11Info>
#define XF86AUDIO_VOLUP 176
#define XF86AUDIO_VOLDN 174
#define XF86AUDIO_MEDIA 129
#define XF86AUDIO_PLAY 162
#define XF86AUDIO_STOP 164
#define XF86AUDIO_NEXT 153
#define XF86AUDIO_PREV 144

extern "C" {
	static int XGrabErrorHandler(Display *, XErrorEvent *e) {
		if ( e->error_code != BadAccess )
			qWarning("grabKey: got X error %d instead of BadAccess", e->type);
		return 0;
	}
}

void QMPDClient::grabKeys() {
	Display *dpy = QX11Info::display();
	XSync(dpy, 0);
	XErrorHandler savedErrorHandler = XSetErrorHandler(XGrabErrorHandler);
	WId rootWindowId = QApplication::desktop()->winId();

	// Grab multimedia keys
	XGrabKey(dpy, XF86AUDIO_PLAY, AnyModifier, rootWindowId, 1, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XF86AUDIO_STOP, AnyModifier, rootWindowId, 1, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XF86AUDIO_PREV, AnyModifier, rootWindowId, 1, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XF86AUDIO_NEXT, AnyModifier, rootWindowId, 1, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XF86AUDIO_MEDIA, AnyModifier, rootWindowId, 1, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XF86AUDIO_VOLUP, AnyModifier, rootWindowId, 1, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XF86AUDIO_VOLDN, AnyModifier, rootWindowId, 1, GrabModeAsync, GrabModeAsync);
	XSync(dpy, 0);
	XSetErrorHandler(savedErrorHandler);
}

void QMPDClient::ungrabKeys() {
	// Ungrab multimedia keys
	Display *dpy = QX11Info::display();
	WId rootWindowId = QApplication::desktop()->winId();
	XUngrabKey(dpy, XF86AUDIO_PLAY, AnyModifier, rootWindowId);
	XUngrabKey(dpy, XF86AUDIO_STOP, AnyModifier, rootWindowId);
	XUngrabKey(dpy, XF86AUDIO_PREV, AnyModifier, rootWindowId);
	XUngrabKey(dpy, XF86AUDIO_NEXT, AnyModifier, rootWindowId);
	XUngrabKey(dpy, XF86AUDIO_MEDIA, AnyModifier, rootWindowId);
	XUngrabKey(dpy, XF86AUDIO_VOLUP, AnyModifier, rootWindowId);
	XUngrabKey(dpy, XF86AUDIO_VOLDN, AnyModifier, rootWindowId);
}

bool QMPDClient::x11EventFilter(XEvent *xevent) {
	XKeyEvent *keyevent = (XKeyEvent *)xevent;

	if (xevent->type != KeyRelease)
		return false;

	switch (keyevent->keycode) {

	case XF86AUDIO_PLAY:
		m_mpd->togglePlay();
		return true;

	case XF86AUDIO_STOP:
		m_mpd->stop();
		return true;

	case XF86AUDIO_NEXT:
		m_mpd->next();
		return true;

	case XF86AUDIO_PREV:
		m_mpd->prev();
		return true;

	case XF86AUDIO_MEDIA:
		m_mainWindow->showHide();
		return true;

	case XF86AUDIO_VOLUP:
		m_mpd->volumeUp();
		return true;

	case XF86AUDIO_VOLDN:
		m_mpd->volumeDown();
		return true;
	}

	return false;
}
#endif

int main(int argc, char **argv) {
	return QMPDClient(argc, argv).exec();
}
